#!/usr/bin/perl

use strict;
use warnings;

use sort 'stable';

my $lurkerconf_fn = "/etc/lurker/lurker.conf";
#my $lurkerconfmm_fn = "/etc/lurker/lurker.conf.mm";
my $lurkerconfmm_fn = "$lurkerconf_fn.mm";
#my $list_lists_fn = "list_lists";
my $list_lists_fn = "/usr/lib/mailman/bin/list_lists |";

open LURKERCONF, $lurkerconf_fn or die "Could not open $lurkerconf_fn: $!";

my (%lists, %grouphead);
$grouphead{undef} = "undef";

my $listhost = "listhost";
my $listurl = "listurl";

my @list_props = qw/title address link description/;

my ($group, $list);
my (@lconfout);

# Parse lurker.conf
while (<LURKERCONF>)
{
    chomp;
    my ($prop,$val);

    if (not (($prop,$val) = /^\s*(\w+)\s*=\s*(.*?)\s*$/))
    {
	push @lconfout, $_;
    }
    elsif ($prop eq 'group')
    {
	$group = $val;
    }
    elsif ($prop eq 'heading')
    {
	$grouphead{$group} = $val;
    }
    elsif ($prop eq 'list')
    {
	$list = $val;
	$lists{$list} = {};
	$lists{$list}{group} = $group;
    }
    elsif (defined $list and grep($prop eq $_, @list_props))
    {
	$lists{$list}{$prop} = $val;
    }
    else
    {
	push @lconfout, $_;
    }
}

close LURKERCONF;

# parse lurkermm.conf

open LURKERMMCONF, $lurkerconfmm_fn or die "Could not open $lurkerconfmm_fn: $!";

while (<LURKERMMCONF>)
{
    if (/\#\s*listhost\s*=\s*(.*?)\s*$/)
    {
	$listhost = $1;
    }
    if (/\#\s*listurl\s*=\s*(.*?)\s*$/)
    {
	$listurl = $1;
    }
}

close LURKERMMCONF;

# parse list_lists

open LIST_LISTS, $list_lists_fn or die "Could not open $list_lists_fn: $!";

while (<LIST_LISTS>)
{
    chomp;
    my ($name, $desc);

    next unless (($name, $desc) = /(\w+) - (.*)$/);

    if ($desc eq "[no description available]")
    {
	$desc = "";
    }

    if (defined $lists{$name})
    {
	if ($desc)
	{
	    $lists{$name}{description} = $desc;
	}
    }
    else
    {
	$lists{$name} = { title => $name,
			  address => $name . '@' . $listhost,
			  link => $listurl . '/' . $name, # todo
			  description => $desc,
			  group => "undef"
		       };
    }
}

close LIST_LISTS;


print join "\n", @lconfout;
print "\n";

# dump groups
$group = "";
for $_ (sort { $lists{$a}{group} cmp $lists{$b}{group} } keys %lists)
{
    unless ($lists{$_}{group} eq $group)
    {
	$group = $lists{$_}{group};
	print "\n";
	print "group = $group\n";
	print "   heading = ", ($grouphead{$group} ? $grouphead{$group} : "undef"), "\n";
	print "\n";
    }

    print "   list = $list\n";
    foreach my $t (@list_props)
    {
	printf "      %-14s = %s\n", $t, ${lists{$_}{$t}};
    }
}
